@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.32 2000/10/06 23:46:44 nisse Exp $ */
+ * $Id: client.h,v 1.33 2000/11/15 21:50:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,6 +34,7 @@
 #include "client.h.x"
 #undef GABA_DECLARE
 
+#if 0
 struct fd_callback *
 make_client_callback(struct io_backend *b,
 		     const char *comment,
@@ -44,6 +45,7 @@ make_client_callback(struct io_backend *b,
 
 struct read_handler *make_client_read_line(struct ssh_connection *c);
 struct lsh_callback *make_client_close_handler(void);
+#endif
 
 struct packet_handler *
 make_accept_service_handler(UINT32 service,
