@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.116 2000/11/14 17:34:44 nisse Exp $ */
+ * $Id: io.c,v 1.118 2000/11/16 22:39:19 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -67,6 +67,45 @@
 
 #include "io.c.x"
 
+/* Calls trigged by a signal handler. */
+/* GABA:
+   (class
+     (name lsh_signal_handler)
+     (super resource)
+     (vars
+       (next object lsh_signal_handler)
+       (flag . "sig_atomic_t *")
+       (action object lsh_callback)))
+*/
+
+/* Scheduled calls. FIXME: For now, no scheduling in time. */
+/* GABA:
+   (class
+     (name lsh_callout)
+     (super resource)
+     (vars
+       (next object lsh_callout)
+       (action object lsh_callback)))
+       ;; (when . time_t)
+*/
+
+/* GABA:
+   (class
+     (name io_backend)
+     (vars
+       ; Linked list of fds. 
+       (files object lsh_fd)
+
+       ; Stack of closed files
+       ;; (closed object lsh_fd)
+
+       ; Flags
+       (signals object lsh_signal_handler)
+       
+       ; Callouts
+       (callouts object lsh_callout)))
+*/
+
 
 /* Backend loop */
 
@@ -85,19 +124,41 @@ int io_iter(struct io_backend *b)
   /* int timeout; */
   int res;
 
+  /* Check all flags */
+  if (b->signals)
+    {
+      struct lsh_signal_handler *f;
+      struct lsh_signal_handler **f_p;
+      for (f_p = &b->signals; (f = *f_p); )
+	{
+	  if (!f->super.alive)
+	    *f_p = f->next;
+	  else
+	    {
+	      if (*f->flag)
+		{
+		  *f->flag = 0;
+		  LSH_CALLBACK(f->action);
+		}
+	      f_p = &f->next;
+	    }
+	}
+    }
+  
   /* Invoke all callouts. Clear the list first; if any callout
    * installs another one, that will not be invoked until the next
    * iteration. */
   if (b->callouts)
     {
-      struct callout *p = b->callouts;
-      b->callouts = NULL;
+      struct lsh_callout *p;
       
-      while (p)
-	{
-	  LSH_CALLBACK(p->action);
-	  p = p->next;
-	}
+      for (p = b->callouts, b->callouts = NULL;
+	   p; p = p->next)
+	if (p->super.alive)
+	  {
+	    LSH_CALLBACK(p->action);
+	    p->super.alive = 0;
+	  }
     }
       
   {
@@ -324,7 +385,8 @@ int io_iter(struct io_backend *b)
 }
 
   
-void io_run(struct io_backend *b)
+void
+io_run(struct io_backend *b)
 {
   struct sigaction pipe;
   memset(&pipe, 0, sizeof(pipe));
@@ -340,12 +402,50 @@ void io_run(struct io_backend *b)
     ;
 }
 
-void init_backend(struct io_backend *b)
+struct io_backend *
+make_io_backend(void)
 {
+  NEW(io_backend, b);
+
   b->files = NULL;
+  b->signals = NULL;
   b->callouts = NULL;
+
+  return b;
 }
 
+struct resource *
+io_signal_handler(struct io_backend *b,
+		  sig_atomic_t *flag,
+		  struct lsh_callback *action)
+{
+  NEW(lsh_signal_handler, handler);
+  resource_init(&handler->super, NULL);
+
+  handler->next = b->signals;
+  handler->flag = flag;
+  handler->action = action;
+
+  b->signals = handler;
+  
+  return &handler->super;
+}
+
+/* Delays not implemented. */
+struct resource *
+io_callout(struct io_backend *b,
+	   UINT32 delay UNUSED,
+	   struct lsh_callback *action)
+{
+  NEW(lsh_callout, callout);
+  resource_init(&callout->super, NULL);
+
+  callout->next = b->callouts;
+  callout->action = action;
+  b->callouts = callout;
+
+  return &callout->super;
+}
 
 /* Read-related callbacks */
 
@@ -767,6 +867,8 @@ static void
 init_file(struct io_backend *b, struct lsh_fd *f, int fd,
 	  struct exception_handler *e)
 {
+  resource_init(&f->super, do_kill_fd);
+
   f->fd = fd;
 
   f->e = make_exception_handler(do_exc_io_handler, e, HANDLER_CONTEXT);
@@ -781,9 +883,6 @@ init_file(struct io_backend *b, struct lsh_fd *f, int fd,
   f->want_write = 0;
   f->write = NULL;
   f->write_close = NULL;
-  
-  f->super.alive = 1;
-  f->super.kill = do_kill_fd;
 
   f->next = b->files;
   b->files = f;
@@ -1877,3 +1976,143 @@ lsh_make_pipe(int *fds)
   return 1;
 }
 
+/* Copies data from one fd to another. Works no matter if the fd:s are
+ * in blocking or non-blocking mode. Tries hard not to do premature
+ * reads; we don't want to read data into the buffer, and then
+ * discover that we can't write it out.
+ *
+ * The src fd may be the process stdin, and if we are too gready
+ * reading it, we may consume data that belongs to the user's next
+ * command. */
+int
+lsh_copy_file(int src, int dst)
+{
+#define BUF_SIZE 1024
+  char buf[BUF_SIZE];
+  struct pollfd src_poll;
+  struct pollfd dst_poll;
+
+  src_poll.fd = src;
+  dst_poll.fd = dst;
+  
+  for (;;)
+    {
+      int res;
+      UINT32 i, length;
+
+      /* First wait until dst is writable; otherwise there's no point
+       * in reading the input. */
+
+      dst_poll.events = POLLOUT;
+
+      do
+	res = poll(&dst_poll, 1, -1);
+      while ( (res < 0) && (errno == EINTR));
+
+      debug("lsh_copy_file, initial poll on destination:\n"
+	    "  res = %i, src = %i, dst = %i, events = %xi, revents = %xi.\n",
+	    res, src, dst, dst_poll.events, dst_poll.revents);
+      
+      assert(res == 1);
+      
+      if (!(dst_poll.revents & POLLOUT))
+	/* Most likely, dst is a pipe, and there are no readers. */
+	return 1;
+
+      /* Ok, can we read anything? */
+      src_poll.events = MY_POLLIN;
+      
+      do
+	res = poll(&src_poll, 1, -1);
+      while ( (res < 0) && (errno == EINTR));
+      
+      if (res < 0)
+	return 0;
+
+      debug("lsh_copy_file, poll on src:\n"
+	    "  res = %i, src = %i, dst = %i, events = %xi, revents = %xi.\n",
+	    res, src, dst, src_poll.events, src_poll.revents);
+      
+      assert(res == 1);
+      
+      /* On linux, it seems that POLLHUP is set on read eof. */
+      if (!(src_poll.revents & MY_POLLIN))
+	/* EOF */
+	return 1;
+
+      assert(src_poll.revents & MY_POLLIN);
+
+      /* Before actually reading anything, we need to check that
+       * the dst fd is still alive. */
+
+      dst_poll.events = POLLOUT;
+      do
+	res = poll(&dst_poll, 1, 0);
+      while ( (res < 0) && (errno == EINTR));
+
+      debug("lsh_copy_file, second poll on destination:\n"
+	    "  res = %i, src = %i, dst = %i, events = %xi, revents = %xi.\n",
+	    res, src, dst, dst_poll.events, dst_poll.revents);
+      
+      if (res && !(dst_poll.revents & POLLOUT))
+	{
+	  /* NOTE: Either somebody else filled up the buffer, or
+	   * the fd is dead. How do we know which happened? We
+	   * can't check POLLHUP, because it seems linux always
+	   * sets it. As a kludge, and because this condition
+	   * should be really rare, we check our ppid to see if
+	   * the main process have died and left us to init. */
+
+	  debug("lsh_copy_file: ppid = %i\n", getppid());
+	  
+	  if (getppid() == 1)
+	    return 1;
+	}
+
+      do 
+	res = read(src, buf, BUF_SIZE);
+      while ( (res < 0) && (errno == EINTR));
+
+      debug("lsh_copy_file: read on fd %i returned = %i\n", src, res);
+      
+      if (res < 0)
+	return 0;
+      else if (!res)
+	/* EOF */
+	return 1;
+
+      length = res;
+
+      for (i = 0; i<length; )
+	{
+	  dst_poll.events = POLLOUT;
+	  do
+	    res = poll(&dst_poll, 1, -1);
+	  while ( (res < 0) && (errno == EINTR));
+
+	  debug("lsh_copy_file, inner poll on destination:\n"
+		"  res = %i, src = %i, dst = %i, events = %xi, revents = %xi.\n",
+		res, src, dst, dst_poll.events, dst_poll.revents);
+      
+	  if (res < 0)
+	    return 0;
+
+	  assert(res == 1);
+	  
+	  if (!(dst_poll.revents & POLLOUT))
+	    return 0;
+
+	  do
+	    res = write(dst, buf + i, length - i);
+	  while ( (res < 0) && (errno == EINTR));
+
+	  debug("lsh_copy_file: write on fd %i returned = %i\n", dst, res);
+	  
+	  if (res < 0)
+	    return 0;
+
+	  i += res;
+	}
+    }
+#undef BUF_SIZE
+}
