@@ -1,6 +1,6 @@
 /* io_commands.h
  *
- * $Id: io_commands.h,v 1.13 2000/04/20 21:08:12 nisse Exp $
+ * $Id: io_commands.h,v 1.14 2000/11/14 17:50:02 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -84,13 +84,14 @@ make_listen_with_callback(struct command *callback,
 extern struct collect_info_1 listen_with_callback;
 #define LISTEN_CALLBACK (&listen_with_callback.super.super.super)
 
+#if 0
 struct command *
 make_listen_with_connection(struct io_backend *backend,
 			    struct ssh_connection *connection);
+#endif
 
 #define LISTEN_CONNECTION (&listen_with_connection.super.super.super)
 
-
 extern struct collect_info_1 connect_with_port;
 #define CONNECT_PORT (&connect_with_port.super.super.super)
 
@@ -111,6 +112,16 @@ struct command *
 make_simple_listen(struct io_backend *backend,
 		   struct resource_list *resources);
 
+struct command *
+make_listen_local(struct io_backend *backend,
+		  struct local_info *info);
+
+struct command *
+make_connect_local(struct io_backend *backend);
+
+extern struct command_simple connect_local_command;
+#define CONNECT_LOCAL (&connect_local_command.super.super)
+
 extern struct command_simple io_log_peer_command;
 #define LOG_PEER (&io_log_peer_command.super.super)
 
