@@ -2,7 +2,7 @@
  *
  * Reads a (private) key on stdin, and saves it a private and a public file.
  *
- * $Id: lsh-writekey.c,v 1.6 2000/11/14 19:54:57 nisse Exp $ */
+ * $Id: lsh-writekey.c,v 1.7 2000/11/15 17:01:41 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -432,13 +432,11 @@ STATIC_EXCEPTION_HANDLER(do_lsh_writekey_handler, NULL);
 
 int main(int argc, char **argv)
 {
-  NEW(io_backend, backend);
+  struct io_backend *backend = make_io_backend();
   struct lsh_writekey_options *options = make_lsh_writekey_options();
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
-  init_backend(backend);
-
   {
     CAST_SUBTYPE
       (command, work,
