@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.121 2000/11/14 19:56:19 nisse Exp $ */
+ * $Id: lsh.c,v 1.124 2000/11/16 22:44:26 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -42,15 +42,14 @@
 #include "io_commands.h"
 #include "gateway.h"
 #include "gateway_commands.h"
+#include "handshake.h"
 #include "lookup_verifier.h"
 #include "randomness.h"
-#include "service.h"
 #include "sexp.h"
 #include "spki_commands.h"
 #include "srp.h" 
 #include "ssh.h"
 #include "tcpforward_commands.h"
-#include "tty.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -118,6 +117,8 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
 
        (random object randomness_with_poll)
 
+       (tty object interact)
+       
        (signature_algorithms object alist)
        (home . "const char *")
        
@@ -162,6 +163,13 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
        (stdin_file . "const char *")
        (stdout_file . "const char *")
        (stderr_file . "const char *")
+
+       ; fork() extra processes for handling stdio file-descriptors,
+       ; to avoid setting them in non-blocking mode.
+       (stdin_fork . int)
+       (stdout_fork . int)
+       (stderr_fork . int)
+       
        ; True if the process's stdin or pty (respectively) has been used. 
        (used_stdin . int)
        (used_pty . int)
@@ -184,6 +192,8 @@ make_options(struct io_backend *backend,
   
   self->backend = backend;
   self->random = make_default_random(NULL, handler);
+
+  self->tty = make_unix_interact(backend);
   
   self->home = getenv("HOME");
   
@@ -211,7 +221,11 @@ make_options(struct io_backend *backend,
   self->stdout_file = NULL;
   self->stderr_file = NULL;
   self->used_stdin = 0;
-    
+
+  self->stdin_fork = 0;
+  self->stdout_fork = 0;
+  self->stderr_fork = 0;
+  
   self->with_pty = -1;
   self->start_shell = 1;
   self->with_remote_peers = 0;
@@ -369,6 +383,7 @@ make_options_command(struct lsh_options *options,
      (super lookup_verifier)
      (vars
        (db object spki_context)
+       (tty object interact)
        (access object sexp)
        (host object address_info)
        ; Allow unauthorized keys
@@ -476,10 +491,12 @@ do_lsh_lookup(struct lookup_verifier *c,
 			  sexp_format(subject->key, SEXP_CANONICAL, 0),
 			  1);
 			  
-	  if (!yes_or_no(ssh_format("Received unauthenticated key for host %lS\n"
-				    "Fingerprint: %lfxS\n"
-				    "Do you trust this key? (y/n) ",
-				    self->host->ip, fingerprint), 0, 1))
+	  if (!INTERACT_YES_OR_NO
+	      (self->tty,
+	       ssh_format("Received unauthenticated key for host %lS\n"
+			  "Fingerprint: %lfxS\n"
+			  "Do you trust this key? (y/n) ",
+			  self->host->ip, fingerprint), 0, 1))
 	    return NULL;
 	}
       
@@ -506,6 +523,7 @@ do_lsh_lookup(struct lookup_verifier *c,
 
 static struct lookup_verifier *
 make_lsh_host_db(struct spki_context *db,
+		 struct interact *tty,
 		 struct address_info *host,
 		 int sloppy,
 		 struct abstract_write *file)
@@ -514,6 +532,7 @@ make_lsh_host_db(struct spki_context *db,
 
   res->super.lookup = do_lsh_lookup;
   res->db = db;
+  res->tty = tty;
   res->access = make_ssh_hostkey_tag(host);
   res->host = host;
   res->sloppy = sloppy;
@@ -533,6 +552,7 @@ do_lsh_verifier(struct command *s,
   CAST(options_command, self, s);
   CAST_SUBTYPE(spki_context, db, a);
   COMMAND_RETURN(c, make_lsh_host_db(db,
+				     self->options->tty,
 				     self->options->remote,
 				     self->options->sloppy,
 				     self->options->capture_file));
@@ -697,7 +717,7 @@ parse_forward_arg(char *arg,
   return 1;
 }
 
-
+#if 0
 /* FIXME: Resetting the tty should really be done by the corresponding
  * channel. */
 
@@ -721,6 +741,7 @@ static int remember_tty(int fd)
 	  && !atexit(reset_tty));
 }
 #endif /* WITH_PTY_SUPPORT */
+#endif
 
 
 /* Option parsing */
@@ -748,6 +769,8 @@ const char *argp_program_bug_address = BUG_ADDRESS;
 #define OPT_STDOUT 0x211
 #define OPT_STDERR 0x212
 
+#define OPT_FORK_STDIO 0x213
+
 static const struct argp_option
 main_options[] =
 {
@@ -802,6 +825,9 @@ main_options[] =
   { "no-stdout", OPT_STDOUT | ARG_NOT, NULL, 0, "Redirect stdout to /dev/null", 0}, 
   { "stderr", OPT_STDERR, "Filename", 0, "Redirect stderr", 0},
   { "no-stderr", OPT_STDERR | ARG_NOT, NULL, 0, "Redirect stderr to /dev/null", 0}, 
+  { "cvs-workaround", OPT_FORK_STDIO, "i?o?e?", OPTION_ARG_OPTIONAL,
+    "fork extra processes to read one or more of the stdio file "
+    "descriptors, to avoid setting them in non-blocking mode.", 0 },
   
 #if WITH_PTY_SUPPORT
   { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
@@ -821,6 +847,61 @@ main_argp_children[] =
   { NULL, 0, NULL, 0}
 };
 
+/* FIXME: Moves to client.c */
+static int
+fork_input(int in)
+{
+  /* pipe[0] for reading, pipe[1] for writing. */
+  int pipe[2];
+
+  if (!lsh_make_pipe(pipe))
+    return -1;
+
+  switch (fork())
+    {
+    case -1:
+      /* Error */
+      return -1;
+    case 0:
+      close(pipe[0]);
+      if (lsh_copy_file(in, pipe[1]))
+	_exit(EXIT_SUCCESS);
+      else
+	_exit(EXIT_FAILURE);
+    default:
+      /* Parent */
+      close(pipe[1]);
+      return pipe[0];
+    }
+}
+
+static int
+fork_output(int out)
+{
+  /* pipe[0] for reading, pipe[1] for writing. */
+  int pipe[2];
+
+  if (!lsh_make_pipe(pipe))
+    return -1;
+
+  switch (fork())
+    {
+    case -1:
+      /* Error */
+      return -1;
+    case 0:
+      close(pipe[1]);
+      if (lsh_copy_file(pipe[0], out))
+	_exit(EXIT_SUCCESS);
+      else
+	_exit(EXIT_FAILURE);
+    default:
+      /* Parent */
+      close(pipe[0]);
+      return pipe[1];
+    }
+}
+
 /* Create a session object. stdout and stderr are shared (although
  * with independent lsh_fd objects). stdin can be used by only one
  * session (until something "session-control"/"job-control" is added).
@@ -832,15 +913,17 @@ make_lsh_session(struct lsh_options *self)
   int out;
   int err;
 
+  debug("lsh.c: Setting up stdin\n");
+
   if (self->stdin_file)
     in = open(self->stdin_file, O_RDONLY);
   else
     {
       if (self->used_stdin)
 	in = open("/dev/null", O_RDONLY);
-      else
+      else 
 	{
-	  in = dup(STDIN_FILENO);
+	  in = (self->stdin_fork ? fork_input : dup)(STDIN_FILENO);
 	  self->used_stdin = 1;
 	}
     }
@@ -852,9 +935,15 @@ make_lsh_session(struct lsh_options *self)
       return NULL;
     }
 
-  out = (self->stdout_file
-	 ? open(self->stdout_file, O_WRONLY | O_CREAT, 0666)
-	 : dup(STDOUT_FILENO));
+  debug("lsh.c: Setting up stdout\n");
+
+  if (self->stdout_file)
+    out = open(self->stdout_file, O_WRONLY | O_CREAT, 0666);
+  else if (self->stdout_fork)
+    out = fork_output(STDOUT_FILENO);
+  else
+    out = dup(STDOUT_FILENO);
+
   if (out < 0)
     {
       werror("lsh: Can't dup/open stdout (errno = %i): %z!\n",
@@ -863,8 +952,12 @@ make_lsh_session(struct lsh_options *self)
       return NULL;
     }
 
+  debug("lsh.c: Setting up stderr\n");
+  
   if (self->stderr_file)
     err = open(self->stderr_file, O_WRONLY | O_CREAT, 0666);
+  else if (self->stderr_fork)
+    err = fork_output(STDERR_FILENO);
   else
     {
       err = dup(STDERR_FILENO);
@@ -893,6 +986,7 @@ make_lsh_session(struct lsh_options *self)
      self->exit_code);
 }
 
+/* FIXME: Moves to client.c */
 /* Create an interactive session */
 static struct command *
 lsh_shell_session(struct lsh_options *self)
@@ -911,18 +1005,18 @@ lsh_shell_session(struct lsh_options *self)
     {
       self->used_pty = 1;
       
-      if (tty_fd < 0)
+      if (self->tty && INTERACT_IS_TTY(self->tty))
 	{
-	  werror("lsh: No tty available.\n");
-	}
-      else
-	{
-	  if (! (remember_tty(tty_fd)
-		 && (get_pty = make_pty_request(tty_fd))))
+	  get_pty = make_pty_request(self->tty);
+	  if (!get_pty)
 	    {
 	      werror("lsh: Can't use tty (probably getattr or atexit() failed.\n");
 	    }
 	}
+      else
+	{
+	  werror("lsh: No tty available.\n");
+	}
     }
 
   get_shell = make_lsh_start_session(&request_shell.super);
@@ -1347,6 +1441,34 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       CASE_ARG(OPT_STDOUT, stdout_file, "/dev/null"); 
       CASE_ARG(OPT_STDERR, stderr_file, "/dev/null");
 
+    case OPT_FORK_STDIO:
+      if (!arg)
+	self->stdin_fork = self->stdout_fork = self->stderr_fork = 1;
+      else
+	{
+	  int i;
+	  for (i = 0; arg[i]; i++)
+	    switch(arg[i])
+	      {
+	      case 'i': case 'I':
+		self->stdin_fork = 1;
+		break;
+	      case 'o': case 'O':
+		self->stdout_fork = 1;
+		break;
+	      case 'e': case 'E':
+		self->stderr_fork = 1;
+		break;
+	      default:
+		argp_error(state, "The argument to --cvs-workaround should "
+			   "be one or more of the characters 'i' (stdin), "
+			   "'o' (stdout) and 'e' (stderr).");
+		goto loop_done;
+	      }
+	loop_done:
+	}
+      break;
+		
     case 'n':
       self->not = !self->not;
       break;
@@ -1437,11 +1559,7 @@ int main(int argc, char **argv)
     = make_lsh_default_handler(&lsh_exit_code, &default_exception_handler,
 			       HANDLER_CONTEXT);
 
-  NEW(io_backend, backend);
-  init_backend(backend);
-
-  /* Attempt to open a tty */
-  lsh_open_tty();
+  struct io_backend *backend = make_io_backend();
   
   /* For filtering messages. Could perhaps also be used when converting
    * strings to and from UTF8. */
