@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.51 2000/10/15 20:41:13 nisse Exp $
+ * $Id: server_session.c,v 1.52 2000/11/15 17:31:40 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -390,8 +390,10 @@ static int make_pty(struct pty_info *pty, int *in, int *out, int *err)
 	  pty->super.alive, pty->master, pty->slave);
   debug("\n");
   
-  if (pty && pty->super.alive)
+  if (pty) 
     {
+      assert(pty->super.alive);
+      
       debug("make_pty: Using allocated pty.\n");
       in[0] = pty->slave;
       in[1] = pty->master;
@@ -881,7 +883,7 @@ do_alloc_pty(struct channel_request *c UNUSED,
 
           if (tty_getattr(pty->slave, &ios))
             {
-	      pty->super.alive = 1;
+	      assert(pty->super.alive);
               session->pty = pty;
 
 	      /* Don't set TERM if the value is empty. */
