@@ -2,7 +2,7 @@
  *
  * Reads a sexp in given form from, and writes it in given form.
  *
- * $Id: sexp-conv.c,v 1.4 2000/10/17 23:46:26 nisse Exp $ */
+ * $Id: sexp-conv.c,v 1.5 2000/11/15 17:35:27 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -384,12 +384,10 @@ int main(int argc, char **argv)
 {
   struct sexp_conv_options *options = make_options();
   struct exception_handler *e;
-  NEW(io_backend, backend);
+  struct io_backend *backend = make_io_backend();
   
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
-  init_backend(backend);
-
   /* Patch the parent pointer later */
   e = make_exception_handler(do_exc_sexp_conv_handler,
 			     NULL, HANDLER_CONTEXT);
